@@ -5,126 +5,58 @@
 
 package Net::LDAPS;
 @Net::LDAPS::ISA = ( 'Net::LDAP' );
-$Net::LDAPS::VERSION = "0.02";
+$Net::LDAPS::VERSION = "0.03";
 
 use strict;
 use Net::LDAP;
-use Net::LDAP::ASN qw(LDAPResponse);
-use Convert::ASN1 qw(asn_read);
 use IO::Socket::SSL;
 
-sub _options {
-    my %r;
-    while (@_) {
-	my ($k, $v) = splice(@_, 0, 2);
-	$r{$k} = $v;
-    }
-    \%r;
-}
-
 # Different OpenSSL verify modes.
 my %verify = qw(none 0 optional 1 require 3);
 
-sub new {
-  my $self = shift;
-  my $type = ref($self) || $self;
-  my $host = shift if @_ % 2;
-  my $arg  = _options(@_);
+sub _connect {
+  my ($ldap, $host, $arg) = @_;
+
+  $ldap->{'net_ldap_socket'} = IO::Socket::SSL->new(
+    PeerAddr 	    => $host,
+    PeerPort 	    => $arg->{'port'} || '636',
+    Proto    	    => 'tcp',
+    Timeout  	    => defined $arg->{'timeout'} ? $arg->{'timeout'} : 120,
+    SSL_context_init_args($arg)
+  );
+}
+
+sub SSL_context_init_args {
+  my $arg = shift;
+
   my $verify = 0;
   my ($clientcert,$clientkey);
-  
+
   if (exists $arg->{'verify'}) {
       my $v = lc $arg->{'verify'};
       $verify = 0 + (exists $verify{$v} ? $verify{$v} : $verify);
   }
+
   if (exists $arg->{'clientcert'}) {
       $clientcert = $arg->{'clientcert'};
       if (exists $arg->{'clientkey'}) {
 	  $clientkey = $arg->{'clientkey'};
       } else {
-	  die "Setting client public key but not client private key";
+	  require Carp;
+	  Carp::croak("Setting client public key but not client private key");
       }
   }
-  my $obj  = bless {}, $type;
 
-  my $sock = IO::Socket::SSL->new(
-				  PeerAddr => $host,
-				  PeerPort => $arg->{'port'} || '636',
-				  Proto    => 'tcp',
-				  Timeout  => defined $arg->{'timeout'}
-				  ? $arg->{'timeout'}
-				  : 120,
-				  SSL_verify_mode => $verify,
-				  SSL_cipher_list => defined $arg->{'ciphers'}
-				  ? $arg->{'ciphers'}
-				  : 'ALL',
-				  SSL_use_cert => $clientcert ? 1 : 0,
-				  SSL_cert_file => $clientcert,
-				  SSL_key_file => $clientcert
-				  ? $clientkey : undef,
-				  SSL_ca_file => exists $arg->{'cafile'}
-				  ? $arg->{'cafile'} : undef,
-				  SSL_ca_path => exists $arg->{'capath'}
-				  ? $arg->{'capath'} : undef,
-				 ) or return;
-
-  $sock->autoflush(1);
-
-  $obj->{'net_ldap_socket'}  = $sock;
-  $obj->{'net_ldap_host'}    = $host;
-  $obj->{'net_ldap_resp'}    = {};
-  $obj->{'net_ldap_debug'}   = $arg->{'debug'} || 0;
-  $obj->{'net_ldap_version'} = $arg->{'version'} || $Net::LDAP::LDAP_VERSION;
-  $obj->{'net_ldap_async'}   = $arg->{'async'} ? 1 : 0;
-
-  $obj;
-}
-
-sub cipher {
-    $_[0]->{'net_ldap_socket'}->get_cipher;
-}
-
-sub certificate {
-    $_[0]->{'net_ldap_socket'}->get_peer_certificate;
-}
-
-# Override a Net::LDAP method because IO::Socket::SSL doesn't support the
-# socket methods (ie send) that Net::LDAP uses.
-
-sub _sendmesg {
-  my $ldap = shift;
-  my $mesg = shift;
-
-  my $debug;
-  if ($debug = $ldap->debug) {
-    require Convert::ASN1::Debug;
-    print STDERR "$ldap sending:\n";
-
-    Convert::ASN1::asn_hexdump(*STDERR, $mesg->pdu)
-      if $debug & 1;
-
-    Convert::ASN1::asn_dump(*STDERR, $mesg->pdu)
-      if $debug & 4;
-  }
-
-  syswrite($ldap->socket, $mesg->pdu, length($mesg->pdu))
-    or return $mesg->set_error(Net::LDAP::LDAP_LOCAL_ERROR,"$!");
-
-  # for CLDAP, here we need to recode when we were sent
-  # so that we can perform timeouts and resends
-
-  my $mid = $mesg->mesg_id;
-
-  unless ($mesg->done) { # may not have a response
-
-    $ldap->{net_ldap_mesg}->{$mid} = $mesg;
-
-    unless ($ldap->async) {
-      my $err = $ldap->sync($mid);
-      $mesg->set_error($err,$@) if $err;
-    }
-  }
-  $mesg;
+  (
+    SSL_cipher_list => defined $arg->{'ciphers'} ? $arg->{'ciphers'} : 'ALL',
+    SSL_ca_file     => exists  $arg->{'cafile'}  ? $arg->{'cafile'}  : '',
+    SSL_ca_path     => exists  $arg->{'capath'}  ? $arg->{'capath'}  : '',
+    SSL_key_file    => $clientcert ? $clientkey : undef,
+    SSL_use_cert    => $clientcert ? 1 : 0,
+    SSL_cert_file   => $clientcert,
+    SSL_verify_mode => $verify,
+    SSL_version     => 'sslv2/3',
+  );
 }
 
 1;
@@ -154,6 +86,12 @@ methods can be used with a Net::LDAPS object; see the documentation
 for Net::LDAP to find out how to query a directory server using the
 LDAP protocol.
 
+Note that the use of LDAPS is not recommended, because it is not
+described by any IETF documents. Instead, you should consider using
+LDAPv3 with the TLS extension defined in RFC 2830. This will give you
+the same functionality as LDAPS, but using recognized standards. See
+L<Net::LDAP/start_tls>.
+
 =head1 CONSTRUCTOR
 
 =over 4
@@ -169,17 +107,19 @@ Net::LDAP::new are:
 =item verify
 
 How to verify the server's certificate, either 'none' (the server may
-provide a cert but don't verify it - this may mean you are be
-connected to the wrong server), 'optional' (verify if the server
-offers a cert), or 'require' (the server must provide a cert, and it
-must be valid.) If you set verify to optional or require, you must
-also set either cafile or capath.
+provide a certificate but it will not be checked - this may mean you
+are be connected to the wrong server), 'optional' (verify if the
+server offers a certificate), or 'require' (the server must provide a
+certificate, and it must be valid.) If you set verify to optional or
+require, you must also set either cafile or capath. The most secure
+option is 'require'.
 
 =item ciphers
 
 Specify which subset of cipher suites are permissible for this
-connection, using the standard OpenSSL string format. The default value
-for ciphers is 'ALL', which permits all ciphers.
+connection, using the standard OpenSSL string format. The default
+value for ciphers is 'ALL', which permits all ciphers, even those that
+don't encrypt!
 
 =item clientcert
 
@@ -188,7 +128,8 @@ for ciphers is 'ALL', which permits all ciphers.
 If you want to use the client to offer a certificate to the server for
 SSL authentication (which is not the same as for the LDAP Bind
 operation) then set clientcert to the user's certificate file, and
-clientkey to the user's private key file.
+clientkey to the user's private key file. These files must be in PEM
+format.
 
 =item capath
 
@@ -197,10 +138,11 @@ clientkey to the user's private key file.
 When verifying the server's certificate, either set capath to the
 pathname of the directory containing CA certificates, or set cafile to
 the filename containing the certificate of the CA who signed the
-server's certificate.
+server's certificate. These certificates must all be in PEM format.
 
 The directory in 'capath' must contain certificates named using the
-hash value of themselves. To generate these names, use OpenSSL thusly:
+hash value of themselves. To generate these names, use OpenSSL like
+this in Unix:
 
     ln -s cacert.pem `openssl x509 -hash -noout < cacert.pem`.0
 
